/*------------------------------------------------------------------------------
| module header files 
------------------------------------------------------------------------------*/
#include "subroutines.h"

/*------------------------------------------------------------------------------
| standard header files 
------------------------------------------------------------------------------*/
#include <math.h>


/*******************************************************************************
* init the matrix A with values
*******************************************************************************/
void initMatrix(double *const A, const int N)
{
  int i,j;
  
  for (i=0; i<N; i++)
  {
    for (j=0; j<N; j++)
      A[j + i*N] = i/(j+1.);
  }
}


/*******************************************************************************
* zero a vector x
*******************************************************************************/
void zeroVector(double *const x, const int N)
{
  int i;
  
  for (i=0; i<N; i++)
    x[i] = 0.;
}


/*******************************************************************************
* init the vector x with values
*******************************************************************************/
void initVector(double *const x, const int N)
{
  int i;
  
  for (i=0; i<N; i++)
    x[i] = i;
}


/*******************************************************************************
* y = alpha*A.x + beta*y
*******************************************************************************/
void matVecMul(const double alpha, const double beta, 
               const double* __restrict__ A,
               const double* __restrict__ x,
               double *const __restrict__ y, const int N)
{
  int i,j;
  
  for (i=0; i<N; i++)
  {
    double h = 0.;
    
    for (j=0; j<N; j++)
    {
       h = h +  A[j + i*N] * x[j];
    }
    y[i] = beta*y[i] + alpha * h;
  }
}


/*******************************************************************************
* calculate the norm of the vector x
*******************************************************************************/
double vecNorm(const double *x, const int N)
{
  int i;
  double nrm2 = 0.0;
  
  for (i=0; i<N; i++)
    nrm2 += pow(x[i], 2);
    
  return sqrt(nrm2);
}


