#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>

#include "timer.h"

static time_t timer;

/* init the timer */
void init_timer()
{
  timer = time(NULL);
}


/* print timer */
void print_timer()
{
  printf("%1.13g [s]\n",difftime(time(NULL),timer));
}




static double system_time(void)
{
        struct timeval tv;
        if(gettimeofday(&tv,NULL) != 0)
                return 0.0 ;
        else
                return (double)tv.tv_sec + 1e-6 * (double)tv.tv_usec;
} /* system_time() */


void msleep(double ms)
{
  double start = system_time();
  if(start>0.001) while(system_time() < start+ms*0.001) {;}
} /* msleep() */


void init_stopwatch(Stopwatch *sw)
{
  sw->user_time = 0.0;
  sw->real_time = 0.0;
  sw->running   = 0;

} /* init_stopwatch() */


void cont_stopwatch(Stopwatch *sw)
{
  if(sw->running)
    return;

  sw->time_start = system_time();
  sw->clock_start = clock();
  sw->running     = 1;

} /* cont_stopwatch() */


void stop_stopwatch(Stopwatch *sw)
{
  if(!sw->running)
    return;

  sw->real_time += system_time() - sw->time_start;
  sw->user_time += (clock() - sw->clock_start) / (CLOCKS_PER_SEC * 1.);
  sw->running    = 0;

} /* stop_stopwatch() */


double read_stopwatch(Stopwatch *sw)
{
        if(!sw->running)
                return sw->real_time;

        return sw->real_time + system_time() -  sw->time_start;

} /* read_stopwatch() */


double read_stopwatch_user(Stopwatch *sw)
{
  if(!sw->running)
    return sw->user_time;

  return sw->user_time + (clock() - sw->clock_start) / (CLOCKS_PER_SEC * 1.);

} /* read_stopwatch_user() */
