/*------------------------------------------------------------------------------
| module header files 
------------------------------------------------------------------------------*/
#include "subroutines.h"

/*------------------------------------------------------------------------------
| standard header files 
------------------------------------------------------------------------------*/
#include <math.h>

#ifdef HAVE_MKL
 #include <mkl.h>
#else
extern void dgemv_(char* trans, int* m, int* n, double* alpha,
        double a[], int* lda, double x[], int* incx, double* beta,
        double y[], int* incy);

extern double dnrm2_(int* n, double x[], int* incx);
#define dnrm2 dnrm2_
#define dgemv dgemv_

#endif

/*******************************************************************************
* init the matrix A with values
*******************************************************************************/
void initMatrix(double *const A, const int N)
{
  int i,j;
  
  for (j=0; j<N; j++)
  {
    const double h = 1.0/(j+1.);
    
    for (i=0; i<N; i++)
      A[i + j*N] = i*h;
  }
}


/*******************************************************************************
* zero a vector x
*******************************************************************************/
void zeroVector(double *const x, const int N)
{
  int i;
  
  for (i=0; i<N; i++)
    x[i] = 0.;
}


/*******************************************************************************
* init the vector x with values
*******************************************************************************/
void initVector(double *const x, const int N)
{
  int i;
  
  for (i=0; i<N; i++)
    x[i] = i;
}


/*******************************************************************************
* y = alpha*A.x + beta*y
*******************************************************************************/
void matVecMul(const double alpha, const double beta, 
               const double* A,
               const double* x,
               double *const y, const int N)
{
  int inc = 1;
  char Trans = 'N';
  
  dgemv(&Trans, &N, &N, &alpha, A, &N, x, &inc ,&beta, y, &inc);
}


/*******************************************************************************
* calculate the norm of the vector x
*******************************************************************************/
double vecNorm(const double *x, const int N)
{
  int inc = 1;

  return dnrm2(&N, x, &inc);
}


