#!/bin/bash

compiler=("gcc" 
          "gcc -O3 -march=native -mtune=native" 
          "gcc -Ofast -IPa -march=native -mtune=native"
          "icc" 
          "icc -O3 -xhost"
          "icc -O3 -xhost -ipo -align"
          "icc -fast -align -xhost"
          "pgcc" 
          "pgcc -O3 -fastsse"
          "opencc"
          "opencc -Ofast -ipa -ffast-math"
          "clang"
          "clang -O3"
          )

modules=("gcc/4.8.1" 
         "gcc/4.8.1" 
         "gcc/4.8.1"
         "ics/2013_sp1.3.174/64" 
         "ics/2013_sp1.3.174/64"
         "ics/2013_sp1.3.174/64" 
         "ics/2013_sp1.3.174/64"
         "pgi/13.10"
         "pgi/13.10"
         "open64/4.5.2.1"
         "open64/4.5.2.1"
         "clang/3.5.0"
         "clang/3.5.0"
         )

EXE=profileTest

# avoid parallel blas
export MKL_NUM_THREADS=1
export MKL_DOMAIN_NUM_THREADS=1
export MKL_DYNAMIC=FALSE

for ((i=0;i<${#compiler[@]};i++))
do
  echo "use <${compiler[i]}>"
  module load ${modules[i]}
  sed -e "s:CC=.*:CC=${compiler[i]}:g" Makefile > myMakefile
  make clean > /dev/null
  make -f myMakefile > /dev/null
  ./$EXE | tee output.txt
  module unload ${modules[i]}
  
  real_time=$(tail -n 1 output.txt | awk '{print $3}'|sed -e 's:,::')
  if [ -z ${reference+x} ]
  then
    reference=$real_time
  fi

  # calculate speedup
  speedup=$(echo "${reference} / ${real_time}" | bc -l)
  echo "Speedup $speedup"
  echo
done

rm output.txt
make clean
