#!/usr/bin/env python


from stopwatch import stopwatch
from math import pow, sqrt
import numpy

#*******************************************************************************
def initMatrix(A, N):
    """
    init the matrix A with values
    """
    A = numpy.fromfunction(lambda i, j: i/(j+1.), (N, N), dtype=float)
    return A

#*******************************************************************************
def zeroVector(x, N):
    """
    zero a vector x
    """
  
    x = 0.*x

#*******************************************************************************
def initVector(x, N):
    """
    init the vector x with values
    """
  
    x = numpy.fromfunction(lambda i: i, (N,), dtype=float)
    return x

#*******************************************************************************
def matVecMul(alpha, beta, A, x, y, N):
    """
    y = alpha*A.x + beta*y
    """

    y = beta*y + alpha * numpy.dot(A, x)

    return y


#*******************************************************************************
def vecNorm(x, N):
    """
    calculate the norm of the vector x
    """

    return numpy.linalg.norm(x)


#*******************************************************************************
def main():
    """
    main routine
    """

    # number of entries
    N = 10000
    
    t_tot = stopwatch()

    #initialize vectors and matrices
    print "create vectors and matrix..."
    x = numpy.zeros(N)
    y = numpy.zeros(N)
    A = numpy.zeros(shape=(N,N))

    # start stop watch 
    t_tot.start()
    
    # zero vectors and matrices 
    print "zero vectors..."
    zeroVector(x, N)
    zeroVector(y, N)

    # init vectors and matrices
    print "initialize vector and matrix..."
    A = initMatrix(A, N)
    x = initVector(x, N)
    
    # perform matrix vector multiplication y = A*x */
    print "matrix/vector multiplication..."
    y = matVecMul(1.0, 0.0, A, x, y, N)
    
    # print norm of result
    print "|A.x| = %g\n" % vecNorm(y, N)

    # stop timer 
    t_tot.stop()
    print "total time:   %7.3gs\n" % t_tot.elapsed
 
    return

################################################################################
# call main
################################################################################
main()
