#!/usr/bin/env python


from stopwatch import stopwatch
from math import pow, sqrt
import array

#*******************************************************************************
def initMatrix(A, N):
    """
    init the matrix A with values
    """
  
    for j in range (0, N):
        h = 1./(j+1.)
        for i in range(0, N):
            A[i + j*N] = h*i

#*******************************************************************************
def zeroVector(x, N):
    """
    zero a vector x
    """
  
    for i in range(0, N):
        x[i] = 0.

#*******************************************************************************
def initVector(x, N):
    """
    init the vector x with values
    """
  
    for i in range(0, N):
        x[i] = i

#*******************************************************************************
def matVecMul(alpha, beta, A, x, y, N):
    """
    y = alpha*A.x + beta*y
    """

    for i in range(0, N):
        y[i] = beta*y[i]
    
    for j in range(0, N):
        for i in range(0, N):
            y[i] += + alpha * A[i + j*N] * x[j]


#*******************************************************************************
def vecNorm(x, N):
    """
    calculate the norm of the vector x
    """
    
    nrm2 = 0.0

    for i in range(0, N):
        nrm2 += x[i]*x[i]
    
    return sqrt(nrm2)


#*******************************************************************************
def main():
    """
    main routine
    """

    # number of entries
    N = 10000
    
    t_tot = stopwatch()

    #initialize vectors and matrices
    x = array.array('d', [0 for i in range(0, N)])
    y = array.array('d', [0 for i in range(0, N)])
    A = array.array('d', [0 for i in range(0, N*N)])

    # start stop watch 
    t_tot.start()
    
    # zero vectors and matrices 
    zeroVector(x, N)
    zeroVector(y, N)
    zeroVector(A, N*N)

    # init vectors and matrices
    initMatrix(A, N)
    initVector(x, N)

    # perform matrix vector multiplication y = A*x */
    matVecMul(1.0, 0.0, A, x, y, N)
    
    # print norm of result
    print "|A.x| = %g\n" % vecNorm(y, N)

    # stop timer 
    t_tot.stop()
    print "total:   %7.3gs\n" % t_tot.elapsed
 
    return

################################################################################
# call main
################################################################################
main()
