/*------------------------------------------------------------------------------
| module header files 
------------------------------------------------------------------------------*/
#include "subroutines.h"

/*------------------------------------------------------------------------------
| standard header files 
------------------------------------------------------------------------------*/
#include <math.h>


/*******************************************************************************
* init the matrix A with values
*******************************************************************************/
void initMatrix(double *const A, const int N)
{
  int i,j;
  
  for (i=0; i<N; i++)
    for (j=0; j<N; j++)
      A[i + j*N] = i/(j+1.);
}


/*******************************************************************************
* zero a vector x
*******************************************************************************/
void zeroVector(double *const x, const int N)
{
  int i;
  
  for (i=0; i<N; i++)
    x[i] = 0.;
}


/*******************************************************************************
* init the vector x with values
*******************************************************************************/
void initVector(double *const x, const int N)
{
  int i;
  
  for (i=0; i<N; i++)
    x[i] = i;
}


/*******************************************************************************
* y = alpha*A.x + beta*y
*******************************************************************************/
void matVecMul(const double alpha, const double beta, 
               const double *A, const double *x, double *const y, const int N)
{
  int i,j;
  
  for (i=0; i<N; i++)
    y[i] = beta*y[i];
    
  for (i=0; i<N; i++)
  {
    for (j=0; j<N; j++)
    {
       y[i] = y[i] +  alpha * A[i + j*N] * x[j];
    }
  }
}


/*******************************************************************************
* calculate the norm of the vector x
*******************************************************************************/
double vecNorm(const double *x, const int N)
{
  int i;
  double nrm2 = 0.0;
  
  for (i=0; i<N; i++)
    nrm2 += pow(x[i], 2);
    
  return sqrt(nrm2);
}


